#include <TkQApplication.h>
#include <time.h>
#include <limits.h>

/* Global variable holding our Qt application instance... */
TkQtApplication *TkQtApplicationInstance = NULL;

/*
 * TkQtApplication Class
 */
TkQtApplication::TkQtApplication(Display *display) :
                  TKQT_APPLICATION(display)
{
  InitialiseTkQt();
  TkQtApplicationInstance = this;
  CurrentStyle            = "default";
  ExecuteWidgetCallback   = NULL;
  interp                  = NULL;
  CommandObj              = NULL;
}

#ifdef TKQT_KDE
TkQtApplication::TkQtApplication(Display *display, int argc, char **argv) :
                  KApplication(display, argc, argv, "TkQt")
#else
TkQtApplication::TkQtApplication(Display *display, int argc, char **argv) :
                  QApplication(display, argc, argv)
#endif
{
  InitialiseTkQt();
  TkQtApplicationInstance = this;
  CurrentStyle            = "default";
  ExecuteWidgetCallback   = NULL;
  interp                  = NULL;
  CommandObj              = NULL;
}

/*
 *
 */
typedef void (*SameAsXtTimerCallbackProc)(void*,void*);
typedef void (*IntervalSetter)(int);
typedef void (*ForeignEventProc)(XEvent*);
typedef void (*XtEventDispatchProc)(XEvent*); 
/* The following functions and variables are defined in qnpsupport.cpp */
extern void     qt_np_add_event_proc(ForeignEventProc fep);
extern int      qt_event_handler(XEvent* event);
extern void     qt_np_add_timeoutcb(SameAsXtTimerCallbackProc cb);
extern void     qt_np_add_timer_setter(IntervalSetter is);
extern int      qt_activate_timers();     // defined in qapplication_x11.cpp
extern timeval *qt_wait_timer();          // defined in qapplication_x11.cpp
extern int      qt_np_timerid;
extern int      qt_np_count;
extern XtEventDispatchProc
 qt_np_cascade_event_handler[LASTEvent];  // defined in qnpsupport.cpp

void TkQtApplication::VisuallyUpdate(void)
{
  XEvent event;
  Tk_Window tkwin;

  tkwin = MainWindow;
  /*
   * Generate an expose event...
   */
  memset((void *) &event, 0, sizeof(event));
  event.xany.type = Expose;
  event.xany.window = Tk_WindowId(tkwin);
  event.xany.display = Tk_Display(tkwin);
  event.xexpose.type = Expose;
  event.xexpose.display = event.xany.display;
  event.xexpose.window = event.xany.window;
  qt_event_handler(&event);
}

/*
 * Various Qt needed functions...
 */
static void np_set_timer(int interval)
{
    // Ensure we only have one timeout in progress - QApplication is
    // computing the one amount of time we need to wait.
    if (qt_np_timerid) {
	//XtRemoveTimeOut( qt_np_timerid );
    }
    //qt_np_timerid = XtAppAddTimeOut(appcon, interval,
    //   (XtTimerCallbackProc)qt_np_timeout, 0);
}
static void np_do_timers(void*,void*)
{
  qt_np_timerid = 0; // It's us, and we just expired, that's why we are here.
  qt_activate_timers();
  timeval *tm = qt_wait_timer();
  if (tm) {
    //int interval = QMIN(tm->tv_sec,INT_MAX/1000)*1000 + tm->tv_usec/1000;
    //np_set_timer( interval );
  }
}
/*
 * This function will deleiver an XEvent received by Tk to Qt. If Qt cannot
 * handle it, it will send it back to Tk...
 */
static long LastQTWindowDeliveredEvent;
void EventHandlerWrapper(ClientData clientData, XEvent *eventPtr)
{
  LastQTWindowDeliveredEvent = (long) eventPtr->xany.window;
  if (eventPtr->xany.window) {
    /*Tk_Window tkwin;
    char *name = "(NULL)";
    tkwin = Tk_IdToWindow(eventPtr->xany.display, eventPtr->xany.window);
    if (tkwin) name = Tk_PathName(tkwin);
    printf("Delivering Event to Qt for window %s (%ld)\n",
           name, eventPtr->xany.window);*/
    qt_event_handler(eventPtr);
    /*
     * Execute any Tcl callbacks registered by widgets...
     */
    TkQtApplicationInstance->ExecuteTclCallabacks();
  } else {
    qt_event_handler(eventPtr);
  }
}
int QtEventHandlerWrapper(ClientData clientData, XEvent *eventPtr)
{
   EventHandlerWrapper(clientData, eventPtr);
   return 0;
}

static void void_event_handler(XEvent *eventPtr)
{
  if (LastQTWindowDeliveredEvent == (long) eventPtr->xany.window) {
    /*
     * If its is about a window we have just sent an event to Qt, we don't
     * want it back :-)
     */
    return;
  }
  if (eventPtr->xany.window) {
    Tk_Window tkwin;
    char *name = "(NULL)";
    tkwin = Tk_IdToWindow(eventPtr->xany.display, eventPtr->xany.window);
    if (tkwin) name = Tk_PathName(tkwin);
    printf("Event got back from Qt for window %s (%ld)\n",
           name, eventPtr->xany.window);
    Tk_HandleEvent(eventPtr);
  }
}

void TkQtApplication::InitialiseTkQt()
{
  /*
   * Inform Qt how to send an event back to Tk, if it fails to handle it...
   */
  for (int et=2; et < LASTEvent; et++) {
    qt_np_cascade_event_handler[et] = void_event_handler;
  }
  qt_np_add_timeoutcb(np_do_timers);
  qt_np_add_timer_setter(np_set_timer);
  qt_np_add_event_proc(void_event_handler/*Tk_HandleEvent*/);
  qt_np_count++;
  //Tk_CreateGenericHandler(QtEventHandlerWrapper, NULL);
}

TkQtApplication::~TkQtApplication()
{
  //Tk_DeleteGenericHandler(EventHandlerWrapper, NULL);
}


