/*
 * This is the TkQt extension.
 * TkQt tries to extend the Tk toolkit with the Qt and KDE toolkits, by
 * providing the ability to mix tk and Qt/KDE under Tcl/Tk.
 *
 * This software (TkQt) is copyrighted by:
 *  George Petasis, National Centre for Scientific Research "Demokritos",
 *  Aghia Paraskevi, Athens, Greece.
 *  e-mail: petasis@iit.demokritos.gr
 *
 * The following terms TkQtApplicationInstancely to all files associated
 * with the software unless explicitly disclaimed in individual files.
 * 
 * The authors hereby grant permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they TkQtApplicationInstancely.
 * 
 * IN NO EVENT SHALL THE AUTHORS OR DISTRIBUTORS BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE, ITS DOCUMENTATION, OR ANY
 * DERIVATIVES THEREOF, EVEN IF THE AUTHORS HAVE BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIM ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT.  THIS SOFTWARE
 * IS PROVIDED ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE
 * NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <TkQApplication.h>
#include <TkQButton.h>
#include <tcl.h>
#include <tk.h>

extern TkQtApplication *TkQtApplicationInstance;
int    TkQt_ExitObjCmd(ClientData dummy, Tcl_Interp *interp,
                       int objc, Tcl_Obj *CONST objv[]);

/*
 * Callback to handle the "::TkQt::qt" command...
 */
int TkQt_qt(ClientData clientData, Tcl_Interp *interp,
            int objc, Tcl_Obj *objv[]) {
  int index;
  static CONST char *Methods[] = {
    "about", "kde", "style", "styles", (char *) NULL
  };
  enum methods {
     ABOUT, QTKDE, STYLE, STYLES
  };
  
  Tcl_ResetResult(interp);
  if (objc < 2) {
    Tcl_WrongNumArgs(interp, 1, objv, "option ?arg ...?");
    return TCL_ERROR;
  }
  if (Tcl_GetIndexFromObj(interp,objv[1],Methods,"method",0,&index) != TCL_OK) 
    return TCL_ERROR;
  switch ((enum methods) index) {
    case ABOUT: {
      if (objc != 2) {
        Tcl_WrongNumArgs(interp, 2, objv, "");
        return TCL_ERROR;
      }
#ifdef TKQT_KDE
      KAboutApplication *aboutKdeDlg;
      aboutKdeDlg = new KAboutApplication(0, "KAbout", 1);
      aboutKdeDlg->exec();
      delete aboutKdeDlg;
      return TCL_OK;
#endif
    }
    case QTKDE: {
      int kde_enabled = 0;
      if (objc != 2) {
        Tcl_WrongNumArgs(interp, 2, objv, "");
        return TCL_ERROR;
      }
#ifdef TKQT_KDE
      kde_enabled = 1;
#endif
      Tcl_SetObjResult(interp, Tcl_NewIntObj(kde_enabled));
      return TCL_OK;
    }
    case STYLE: {
      char *style_name;
      if (objc != 2 && objc != 3) {
        Tcl_WrongNumArgs(interp, 2, objv, "?style?");
        return TCL_ERROR;
      }
      if (objc == 2) {
        /* Return the current style */
        Tcl_SetResult(interp, TkQtApplicationInstance->StyleDescription(),
                      TCL_STATIC);
        return TCL_OK;
      }   
      style_name = Tcl_GetString(objv[2]);
      if (strcmp(style_name, "KDE") == 0) {
        TkQtApplicationInstance->StyleDescription("KDE");
        TkQtApplicationInstance->setStyle(TkQtApplicationInstance->kstyle());
      } else {
        TkQtApplicationInstance->StyleDescription(style_name);
        TkQtApplicationInstance->setStyle(QStyleFactory::create(style_name));
        TkQtApplicationInstance->VisuallyUpdate();
      }
      return TCL_OK;
#ifdef TKQT_KDE
      if (strcmp(style_name, "KDE") == 0) {
        TkQtApplicationInstance->StyleDescription("KDE");
        TkQtApplicationInstance->setStyle(new KStyle);
      }
#endif
      TkQtApplicationInstance->VisuallyUpdate();
      return TCL_OK;
    }
    case STYLES: {
      QStringList list = QStyleFactory::keys(); 
      Tcl_Obj *styles;
      if (objc != 2) {
        Tcl_WrongNumArgs(interp, 2, objv, "");
        return TCL_ERROR;
      }
      list.sort();
      styles = Tcl_NewListObj(0, NULL);
      if (styles == NULL) return TCL_ERROR;
      for (QStringList::Iterator it = list.begin(); it != list.end(); ++it ) {
        QString styleStr = *it;
        QCString utf8 = styleStr.utf8();
        Tcl_ListObjAppendElement(interp, styles, 
                                 Tcl_NewStringObj((const char *) utf8, -1));
      }
      Tcl_SetObjResult(interp, styles);
      return TCL_OK;
    }
  }
  return TCL_OK;
} /* TkQt_qt */

/*
 * Initialise the TkQt package...
 */
extern "C" int Tkqt_Init(Tcl_Interp *interp) {
  static char *argv[2];
  char buf[256];
  argv[0] = "TkQt";
  argv[1] = "wish";
  /*
   * Use the stubs mechanism so as not to depend on the specific Tcl version.
   * Doing so will enable the binaries compiled under one system to run on
   * other systems, even if a different version of Tcl is installed.
   * Note that we pose a limit of the lowest version that can be used.
   * In order to run, we require a Tcl version >= 8.1...
   */
#ifdef USE_TCL_STUBS 
  if (Tcl_InitStubs(interp, "8.1", 0) == NULL) {
    return TCL_ERROR;
  }
  if (Tk_InitStubs(interp, "8.1", 0) == NULL) {
    return TCL_ERROR;
  }
#endif /* CDM_USE_STUBS */

  /*
   * Well, we need Tk in order to work :-)
   */
  if (Tcl_PkgRequire(interp, "Tk", "8.1", 0) == NULL) {
    return TCL_ERROR;
  }
  /*
   * Initialise a Qt TkQtApplicationInstancelication object (but only once!)...
   */
  if (TkQtApplicationInstance == NULL) {
#ifdef TKQT_KDE
    static KAboutData aboutData("TkQt",I18N_NOOP("TkQt"), "0.0.1.beta :-)",
                       I18N_NOOP("TkQt is a Tcl/Tk extension that embeds\n"
                       "Qt/KDE widgets inside Tk."),
                       KAboutData::License_BSD, 
                       "Copyright 2001 by George Petasis",
                       0,
                       "http://www.iit.demokritos.gr/~petasis",
                       "petasis@iit.demokritos.gr");
    KCmdLineArgs::init(1, argv, &aboutData);
#endif
    new TkQtApplication(Tk_Display(Tk_MainWindow(interp)));
    TkQtApplicationInstance->enableStyles();
    TkQtApplicationInstance->MainInterp = interp;
    TkQtApplicationInstance->MainWindow = Tk_MainWindow(interp);
  }
  
  /* 
   * Register the Module Main Command 
   */
  strcpy(buf, "namespace eval ::TkQt { namespace export qt }");
  if (Tcl_Eval(interp, buf) != TCL_OK) return TCL_ERROR;
  if (Tcl_CreateObjCommand(interp, "::TkQt::qt", (Tcl_ObjCmdProc*) TkQt_qt,
           (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL) == NULL) {
    return TCL_ERROR;
  }

  /*
   * Register widgets
   */
  strcpy(buf, "namespace eval ::TkQt { namespace export button }");
  if (Tcl_Eval(interp, buf) != TCL_OK) return TCL_ERROR;
  if (Tcl_CreateObjCommand(interp, "::TkQt::button",
         (Tcl_ObjCmdProc*) TkQt_button, (ClientData) NULL,
         (Tcl_CmdDeleteProc *) NULL) == NULL) return TCL_ERROR;
  if (Tcl_CreateObjCommand(interp, "::TkQt::exit",
         (Tcl_ObjCmdProc*) TkQt_ExitObjCmd, (ClientData) NULL,
         (Tcl_CmdDeleteProc *) NULL) == NULL) return TCL_ERROR;

  return TCL_OK;
} /* Creole_tkqt_Init */

/*
 *----------------------------------------------------------------------
 *
 * TkQt_ExitObjCmd --
 *
 *        This procedure is invoked to process the "exit" Tcl command.
 *        See the user documentation for details on what it does.
 *
 * Results:
 *        A standard Tcl object result.
 *
 * Side effects:
 *        See the user documentation.
 *
 *----------------------------------------------------------------------
 */

        /* ARGSUSED */
int
TkQt_ExitObjCmd(ClientData dummy, Tcl_Interp *interp,
                int objc, Tcl_Obj *CONST objv[]) {
  int value;

  if ((objc != 1) && (objc != 2)) {
      Tcl_WrongNumArgs(interp, 1, objv, "?returnCode?");
      return TCL_ERROR;
  }
  
  if (objc == 1) {
      value = 0;
  } else if (Tcl_GetIntFromObj(interp, objv[1], &value) != TCL_OK) {
      return TCL_ERROR;
  }
  printf("Exiting Qt with %d\n", value);
  TkQtApplicationInstance->exit(value);
  printf("Exiting Tcl with %d\n", value);
  Tcl_Exit(value);
  /*NOTREACHED*/
  return TCL_OK;                        /* Better not ever reach this! */
}
/* 
 * End of TkQt.c
 */

